@@ -1,8 +1,11 @@
 cat2pal <- function(x, 
+					var,
 					palette = "Set3",
-					auto.palette.mapping = TRUE,
+					stretch.palette = TRUE,
+					#auto.palette.mapping = TRUE,
 					contrast = 1, 
 					colorNA = "#FF1414",
+					colorNULL = "#FFFFFF",
 					legend.labels = NULL,
 					max_levels = 40,
 					legend.NA.text = "Missing",
@@ -10,16 +13,24 @@ cat2pal <- function(x,
 					process.colors,
 					legend.format=list(align="left"),
 					reverse=FALSE) {
+	
+	sel <- attr(x, "sel")
+	if (is.null(sel)) sel <- rep(TRUE, length(x))
+	
+	x[!sel] <- NA
+	
 	if (!is.factor(x)) {
 		su <- sort(unique(x))
-		if (is.numeric(su) && length(su) > max_levels) stop("Number of unique values is more than max.categories, so style = \"cat\" cannot be used. Please use numeric intervals instead, e.g. with style =  \"pretty\"")
+		if (is.numeric(su) && length(su) > max_levels) stop("Number of unique values of the variable \"", var, "\" is ", length(su), ", which is more than max.categories (which is ", max_levels, "), so style = \"cat\" cannot be used. Please use numeric intervals instead, e.g. with style =  \"pretty\".")
 		x <- factor(x, levels=su)
+		if (is.numeric(su)) levels(x) <- do.call("fancy_breaks", c(list(vec=su, intervals=FALSE), legend.format)) 	
 	}
 	
 	
 	# quick&dirty
 	nCol <- nlevels(x)
-	if (nCol > max_levels && !auto.palette.mapping) {
+	if (nCol > max_levels) {
+		warning("Number of levels of the variable \"", var ,"\" is ", nCol, ", which is larger than max.categories (which is ", max_levels, "), so levels are combined. Set tmap_options(max.categories = ", nCol, ") in the layer function to show all levels.", call. = FALSE)
 	
 		mapping <- as.numeric(cut(seq.int(nCol), breaks=max_levels))
 		to <- c(which(mapping[-nCol] - mapping[-1]!=0), nCol)
@@ -39,10 +50,15 @@ cat2pal <- function(x,
 	} else revPal <- function(p)p
 	
 	
-	legend.palette <- if (palette[1] %in% rownames(brewer.pal.info)) {
-		revPal(suppressWarnings(get_brewer_pal(palette, nCol, contrast, stretch = auto.palette.mapping, plot = FALSE)))
+	legend.palette <- if (palette[1] %in% rownames(tmap.pal.info)) {
+		if (tmap.pal.info[palette[1], "origin"] == "brewer") {
+			revPal(suppressWarnings(get_brewer_pal(palette[1], nCol, contrast, stretch = stretch.palette, plot = FALSE)))			
+		} else {
+			# viridis palette
+			revPal(viridis(nCol, option = palette[1]))
+		}
 	} else {
-		if (auto.palette.mapping && (length(palette) < nCol)) {
+		if (stretch.palette && (length(palette) < nCol)) {
 			colorRampPalette(palette)(nCol)	
 		} else rep(palette, length.out=nCol)
 	}
@@ -64,11 +80,12 @@ cat2pal <- function(x,
 		legend.labels <- rep(legend.labels, length.out = nCol)
 	}
 	if (any(colsNA)) {
-		if (is.na(showNA)) showNA <- TRUE
+		if (is.na(showNA)) showNA <- any(colsNA & sel)
 		cols[colsNA] <- colorNA
 	} else {
 		if (is.na(showNA)) showNA <- FALSE
 	}
+	cols[!sel] <- colorNULL
 	
 	legend.values <- legend.labels
 	
