@@ -117,14 +117,7 @@ gridplot <- function(gmeta, fun, nx, gps, gal, shps, dasp, sasp, inner.margins.n
 				}
 				gt <- gps[[i]]$tm_layout
 				if (gt$grid.show) {
-					# non inverse projection avaiable PROJ.4 4.8.0 for Winkel Tripel projection
-					PROJ4_version_nr <- get_proj4_version()
-					if (length(grep("+proj=wintri", proj, fixed = TRUE)) && PROJ4_version_nr < 491 && !is.na(gt$grid.projection)) {
-						warning("Unable to reproject grid lines from the Winkel Triple projection with PROJ.4 version < 4.9.1", call. = FALSE)
-						gt$grid.show <- FALSE
-					} else {
-						gt <- process_grid(gt, bbx, proj, sasp)
-					}
+					gt <- process_grid(gt, bbx, proj, sasp)
 				}
 				gTree(children=gList(
 					cellplot((rw+1):nrw,cl, e=plot_grid_labels_x(gt, scale=gt$scale), name="gridLabelsX"),
