@@ -4,7 +4,10 @@ plot_map <- function(i, gp, gt, shps, bbx, proj, sasp) {
 	## bubble height needed to align with bubbles in legend
 	lineInch <- convertHeight(unit(1, "lines"), "inch", valueOnly=TRUE)
 	
+	lineNatH <- convertHeight(unit(lineInch, "inch"), "npc", valueOnly=TRUE) * (bbx[4] - bbx[2])
+	lineNatW <- convertWidth(unit(lineInch, "inch"), "npc", valueOnly=TRUE) * (bbx[3] - bbx[1])
 	
+
 	## grid lines
 	## metaX and Y are X and Y margins for the meta plot (legend etc)
 	if (gt$grid.show) {
@@ -23,28 +26,35 @@ plot_map <- function(i, gp, gt, shps, bbx, proj, sasp) {
 
 	## thematic map layers
 	treeElements <- mapply(function(gpl, shp, k) {
-		if (length(shp)==0) return(NULL)
+		if (length(shp) == 0 || nrow(shp) == 0) return(NULL)
 		
 		bbx <- attr(shp, "bbox")
 		
 		## obtain coordinates (to draw bubbles and text)
-		if (inherits(shp, "Spatial")) {
-			res <- get_sp_coordinates(shp, gpl, gt, bbx)
-			co.npc <- res$co
-			if (gt$shape.line.center.type[1]=="segment") {
+		if (inherits(shp, "sf")) {
+			#co.native <- suppressWarnings(st_coordinates(st_geometry(st_centroid(shp)))) #st_coordinates(shp)
+			res <- get_sf_coordinates(shp, gpl)
+			co.native <- res$co
+			if (attr(shp, "point.per")=="segment") {
 				gpl <- res$gpl
 				shp <- res$shp
-			}	
-			co.npc[,1] <- if (bbx[1, 2]-bbx[1,1]==0) .5 else {
-				(co.npc[,1]-bbx[1,1]) / (bbx[1, 2]-bbx[1,1])	
-			}
-			co.npc[,2] <- if (bbx[2, 2]-bbx[2,1]==0) .5 else {
-				(co.npc[,2]-bbx[2,1]) / (bbx[2, 2]-bbx[2,1])
 			}
+			
+			# co.native <- suppressWarnings(st_coordinates(st_geometry(st_centroid(shp)))) #st_coordinates(shp)
+			
+			# co.npc[,1] <- if (bbx[1, 2]-bbx[1,1]==0) .5 else {
+			# 	(co.npc[,1]-bbx[1,1]) / (bbx[1, 2]-bbx[1,1])	
+			# }
+			# co.npc[,2] <- if (bbx[2, 2]-bbx[2,1]==0) .5 else {
+			# 	(co.npc[,2]-bbx[2,1]) / (bbx[2, 2]-bbx[2,1])
+			# }
 		} else {
-			co.npc <- NA
+			# co.npc <- NA
+			co.native <- NA
 		}
 
+		plot_tm_tiles <- function() NULL
+		
 		plot_tm_fill <- function() {
 			fill <- if (is.null(gpl$fill)) NA else gpl$fill
 			col <- gpl$col
@@ -57,8 +67,8 @@ plot_map <- function(i, gp, gt, shps, bbx, proj, sasp) {
 									   lineend="butt"), i, k)
 		}
 		
-		plot_tm_symbols <- function() plot_symbols(co.npc, gpl, gt, lineInch, i, k)
-		plot_tm_text <- function() plot_text(co.npc, gpl, gt, lineInch, just=gpl$text.just)
+		plot_tm_symbols <- function() plot_symbols(co.native, gpl, gt, lineInch, lineNatH, lineNatW, i, k)
+		plot_tm_text <- function() plot_text(co.native, gpl, gt, lineNatH, lineNatW, just=gpl$text.just, bbx=bbx)
 		
 		
 		plot_tm_grid <- function() treeGridLines
@@ -66,21 +76,22 @@ plot_map <- function(i, gp, gt, shps, bbx, proj, sasp) {
 		plot_tm_raster <- function() {
 			rast <- if (is.null(gpl$raster)) NA else gpl$raster
 			bb_target <- attr(shp, "bbox")
-			bb_real <- bbox(shp)
+			bb_real <- bb(shp)
 			
 			if (all(abs(bb_real-bb_target)< 1e-3)) {
 				width <- 1
 				height <- 1
-				cent <- rowMeans(bb_target)
+				cent <- c(mean(c(bb_target[1], bb_target[3])), mean(c(bb_target[2], bb_target[4])))
 			} else {
-				width <- (bb_real[1,2] - bb_real[1,1]) / (bb_target[1,2] - bb_target[1,1])
-				height <- (bb_real[2,2] - bb_real[2,1]) / (bb_target[2,2] - bb_target[2,1])
-				cent <- rowMeans(bb_real)
+				width <- (bb_real[3] - bb_real[1]) / (bb_target[3] - bb_target[1])
+				height <- (bb_real[4] - bb_real[2]) / (bb_target[4] - bb_target[2])
+				cent <- c(mean(c(bb_real[1], bb_real[3])), mean(c(bb_real[2], bb_real[4])))
 			}
 			
-			x <- (cent[1] - bb_target[1,1]) / (bb_target[1,2] - bb_target[1,1])
-			y <- (cent[2] - bb_target[2,1]) / (bb_target[2,2] - bb_target[2,1])
+			x <- (cent[1] - bb_target[1]) / (bb_target[3] - bb_target[1])
+			y <- (cent[2] - bb_target[2]) / (bb_target[4] - bb_target[2])
 			#if (inherits(shp, "Spatial")) shp <- as(shp, "RasterLayer")
+
 			rasterGrob(matrix(rast, ncol=shp@ncols, nrow=shp@nrows, byrow = TRUE), x=x, y=y, width=width, height=height, interpolate = gpl$raster.misc$interpolate)
 		} 
 		
@@ -95,59 +106,61 @@ plot_map <- function(i, gp, gt, shps, bbx, proj, sasp) {
 			tGrob <- grobs[[which(fnames=="plot_tm_text")]]
 			if (!is.null(tGrob[[1]])) {
 				tG <- tGrob[[1]]
-				
-				tGX <- convertX(tG$x, "npc", valueOnly = TRUE)
-				tGY <- convertY(tG$y, "npc", valueOnly = TRUE)
-				#tGWidth <- convertWidth(tG$width, "npc", valueOnly = TRUE)
-				#tGHeight <- convertHeight(tG$height, "npc", valueOnly = TRUE)
+
+				tGX <- convertX(tG$x, "native", valueOnly = TRUE)
+				tGY <- convertY(tG$y, "native", valueOnly = TRUE)
+				#tGWidth <- convertWidth(tG$width, "native", valueOnly = TRUE)
+				#tGHeight <- convertHeight(tG$height, "native", valueOnly = TRUE)
 				nt <- length(tGX)
-				
-				
+
+				deltax <- bbx[3] - bbx[1]
+				deltay <- bbx[4] - bbx[2]
+				delta <- max(deltax, deltay)
+
 				coords <- cbind(tGX, tGY)
 				if (gpl$text.along.lines && "plot_tm_lines" %in% fnames) {
 					lGrob <- grobs[[which(fnames=="plot_tm_lines")]]
-					lShp <- polylineGrob2Lines(lGrob)
+					lShp <- polylineGrob2sfLines(lGrob)
+
+					# lShps <- lapply(lShp@lines, function(l){
+					# 	SpatialLines(list(l), proj4string = lShp@proj4string)
+					# })
 					
-					lShps <- lapply(lShp@lines, function(l){
-						SpatialLines(list(l), proj4string = lShp@proj4string)
-					})
+					pShp <- st_as_sf(as.data.frame(coords), coords = c("tGX", "tGY"))
 					
-					pShp <- SpatialPoints(coords, proj4string = lShp@proj4string)
-					ppShp <- as(gBuffer(pShp, width = .01, byid = TRUE), "SpatialLines")
-					ppShps <- lapply(ppShp@lines, function(l){
-						SpatialLines(list(l), proj4string = ppShp@proj4string)
-					})
-					iShps <- mapply(gIntersection, lShps, ppShps, MoreArgs = list(byid = FALSE))
 					
-					angles <- sapply(iShps, function(x) {
-						if (is.null(x)) 0 else {
-							if (inherits(x, "SpatialPoints")) {
-								.get_direction_angle(x@coords)	
-							} else if (inherits(x, "SpatialLines")) {
-								.get_direction_angle(x@lines[[1]]@Lines[[1]]@coords)	
-							} else 0
-							
-						}
-					})
+					# plot(lShp)
+					# plot(pShp, add=TRUE, pch=19)
+					# 
+					# plot(iShps[[2]], add=TRUE, col="red")
+					
+					pbShp <- st_cast(st_geometry(st_buffer(pShp, dist = delta / 100)), "MULTILINESTRING")
+
+					# plot(pbShp, add=TRUE)
+					
+					iShps <- mapply(st_intersection, lShp, pbShp)
+
+					angles <- vapply(iShps, function(x) {
+						if (length(x) == 0) 0 else .get_direction_angle(st_coordinates(x)[,1:2])
+					}, numeric(1))
 				} else angles <- rep(0, nt)
-				
 
 				rG <- if (any(angles!=0)) {
 					.rectGrob2pathGrob(tGrob[[1]], angles)$rect
 				} else tG
-				
-				rGX <- convertX(rG$x, "npc", valueOnly = TRUE)
-				rGY <- convertY(rG$y, "npc", valueOnly = TRUE)
-				rGWidth <- convertWidth(rG$width, "npc", valueOnly = TRUE)
-				rGHeight <- convertHeight(rG$height, "npc", valueOnly = TRUE)
-				
-				
+
+				rGX <- convertX(rG$x, "native", valueOnly = TRUE)
+				rGY <- convertY(rG$y, "native", valueOnly = TRUE)
+				rGWidth <- convertWidth(rG$width, "native", valueOnly = TRUE)
+				rGHeight <- convertHeight(rG$height, "native", valueOnly = TRUE)
+
+
 				# Automatic label placement (Simulated Annealing)
 				if (gpl$text.auto.placement || identical(gpl$text.auto.placement, 0)) {
 					el <- if (is.numeric(gpl$text.auto.placement)) gpl$text.auto.placement * .5 else 0
 					textSizes <- gpl$text.size[gpl$text_sel]
-					elX <- convertWidth(unit(textSizes, "lines"), "npc", valueOnly = TRUE) * el
-					elY <- convertHeight(unit(textSizes, "lines"), "npc", valueOnly = TRUE) * el
+					elX <- convertWidth(unit(textSizes, "lines"), "native", valueOnly = TRUE) * el
+					elY <- convertHeight(unit(textSizes, "lines"), "native", valueOnly = TRUE) * el
 					xy <- pointLabelGrid(rGX-elX*.5, rGY-elY*.5, rGWidth+elX, rGHeight+elY, xyAspect = sasp)
 					dir <- atan2(xy$y - rGY, xy$x - rGX)
 					shiftX <- (xy$x - rGX) + elX * cos(dir)
@@ -171,7 +184,7 @@ plot_map <- function(i, gp, gt, shps, bbx, proj, sasp) {
 						x2 <- x + w/2
 						y1 <- y - h/2
 						y2 <- y + h/2
-						
+
 						seli <- sel
 						seli[i] <- FALSE
 						xr <- rGX2[seli]
@@ -182,13 +195,13 @@ plot_map <- function(i, gp, gt, shps, bbx, proj, sasp) {
 						xr2 <- xr + wr/2
 						yr1 <- yr - hr/2
 						yr2 <- yr + hr/2
-						
+
 						coverx <- ((x1 > xr1) & (x1 < xr2)) | ((x2 > xr1) & (x2 < xr2))
 						covery <- ((y1 > yr1) & (y1 < yr2)) | ((y2 > yr1) & (y2 < yr2))
 						sel[i] <- !any(coverx & covery)
 					}
 				}
-				
+				#pushViewport(viewport(xscale = bbx[c(1,3)], yscale = bbx[c(2,4)]))
 				# redo automatic labeling (with selection)
 				if (gpl$text.auto.placement && (!all(sel))) {
 					shiftX <- rep(0, nt)
@@ -200,46 +213,43 @@ plot_map <- function(i, gp, gt, shps, bbx, proj, sasp) {
 					rGY2 <- rGY2 + shiftY
 				}
 				tGrob <- do.call("gList", lapply(tGrob, .editGrob, sel=sel, shiftX=shiftX, shiftY=shiftY, angles=angles))
-				
+
 				if (gpl$text.overwrite.lines && "plot_tm_lines" %in% fnames) {
 					# Remove line where labels overlap
 					lGrob <- grobs[[which(fnames=="plot_tm_lines")]]
-					
+
 					tShp <- .grob2Poly(tGrob[[1]])
+					lShp <- polylineGrob2sfLines(lGrob)
+
+					lShp$tempID <- 1:nrow(lShp)
+					#ids <- as.character(shp$tempID)
 					
-					lShp <- polylineGrob2Lines(lGrob)
+					dShp <- suppressWarnings(st_difference(lShp, tShp))
 					
-					shp$tempID <- 1:length(shp)
-					ids <- as.character(shp$tempID)
-					dShp <- gDifference(lShp, tShp, byid = TRUE, id=ids)
-					ids <- as.character(shp$tempID) #bug in gDifference: ids changes
 					
-					matched <- match(get_IDs(dShp), ids)
-					nonmatched <- match(setdiff(ids, get_IDs(dShp)),ids)
+					nonmatched <- setdiff(1:nrow(lShp), dShp$tempID)
 					
-					dShp <- sbind(dShp, lShp[nonmatched, ])
-					allmatched <- match(get_IDs(dShp),ids)
-					lco <- coordinates(dShp)
+					lShp$geometry[dShp$tempID] <- dShp$geometry
 					
-					sel2 <- (!(1:length(shp)) %in% nonmatched)[sel]
+
+					sel2 <- ((1:nrow(lShp)) %in% dShp$tempID)[sel]
 					
-					lGrobSel <- lGrob[allmatched]
-					lGrob_new <- do.call("gList", mapply(function(lG, lC) {
-						nr <- sapply(lC, nrow)
-						coor <- do.call("rbind", lC)
-						lG$x <- unit(coor[,1], "npc")
-						lG$y <- unit(coor[,2], "npc")
-						lG$id <- do.call("c", mapply(rep, 1:length(lC), nr, SIMPLIFY=FALSE))
+					lco <- st_coordinates(st_cast(lShp, do_split = TRUE))
+
+					lGrob_new <- do.call("gList", mapply(function(lG, i) {
+						coor <- lco[lco[,4] == i,]
+						lG$x <- unit(coor[,1], "native")
+						lG$y <- unit(coor[,2], "native")
+						lG$id <- coor[,3]
 						lG
-					}, lGrobSel, lco, SIMPLIFY=FALSE))
+					}, lGrob, 1:nrow(lShp), SIMPLIFY=FALSE))
 
 					tGrob <- do.call("gList", lapply(tGrob, .editGrob, sel=sel2, shiftX=0, shiftY=0, angles=angles[sel]))
-					
 					grobs[[which(fnames=="plot_tm_lines")]] <- lGrob_new
 				}
 
-				
-				
+
+
 				# remove unused background
 				grobs[[which(fnames=="plot_tm_text")]] <- if (is.na(tGrob[[1]]$gp$fill)) {
 					tGrob[-1]
@@ -257,10 +267,11 @@ plot_map <- function(i, gp, gt, shps, bbx, proj, sasp) {
 	if (gt$earth.boundary) {
 		world_bb_sp2 <- tmaptools::bb_earth(projection = proj, earth.datum = gt$earth.datum, bbx = gt$earth.bounds)
 		if (!is.null(world_bb_sp2)) {
-			world_bb_co2 <- world_bb_sp2@polygons[[1]]@Polygons[[1]]@coords
 			
-			world_bb_co3 <- matrix(c((world_bb_co2[,1] - bbx[1,1])/(bbx[1,2]-bbx[1,1]),
-									 (world_bb_co2[,2] - bbx[2,1])/(bbx[2,2]-bbx[2,1])), ncol=2)
+			world_bb_co2 <- sf::st_coordinates(world_bb_sp2)
+			
+			world_bb_co3 <- matrix(c((world_bb_co2[,1] - bbx[1])/(bbx[3]-bbx[1]),
+									 (world_bb_co2[,2] - bbx[2])/(bbx[4]-bbx[2])), ncol=2)
 			
 			worldBB_bg <- if (!is.na(gt$frame)) NULL else pathGrob(x = world_bb_co3[,1], y = world_bb_co3[,2], id=rep(1,nrow(world_bb_co3)), gp=gpar(col=NA, fill=gt$bg.color))
 			worldBB <- pathGrob(x = world_bb_co3[,1], y = world_bb_co3[,2], id=rep(1,nrow(world_bb_co3)), gp=gpar(col=gt$earth.boundary.color, fill=NA, lwd=gt$earth.boundary.lwd))
